"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = createGenerator;

var _utils = require("@elodin/utils");

var _cssInJsUtils = require("css-in-js-utils");

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function createGenerator() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      adapter = _ref.adapter,
      _ref$devMode = _ref.devMode,
      devMode = _ref$devMode === void 0 ? false : _ref$devMode,
      _ref$rootNode = _ref.rootNode,
      rootNode = _ref$rootNode === void 0 ? 'body' : _ref$rootNode,
      _ref$dynamicImport = _ref.dynamicImport,
      dynamicImport = _ref$dynamicImport === void 0 ? false : _ref$dynamicImport;

  var config = {
    devMode: devMode,
    rootNode: rootNode,
    dynamicImport: dynamicImport
  };

  if (!adapter) {
    throw new Error('An adapter needs to passed in order to generate code. See @elodin/generator-css-in-js/lib/adapters for more information.');
  }

  return function generate(ast, fileName) {
    var css = generateCSS(ast, config);
    var js = generateJS(ast, config, adapter);
    var root = generateRoot(ast, config);
    return _objectSpread(_defineProperty({}, fileName + '.js', root), css, {}, js);
  };
}

function generateRoot(ast) {
  // TODO: include fragments
  var styles = ast.body.filter(function (node) {
    return node.type === 'Style';
  });
  var imports = styles.map(function (module) {
    return 'import { ' + module.name + " } from './" + module.name + ".elo.js'";
  }).join('\n');
  return imports + '\n\n' + 'export {\n  ' + styles.map(function (module) {
    return module.name;
  }).join(',\n  ') + '\n}';
}

function generateCSS(ast, _ref2) {
  var devMode = _ref2.devMode;
  // TODO: include fragments
  var styles = ast.body.filter(function (node) {
    return node.type === 'Style';
  });
  var variants = ast.body.filter(function (node) {
    return node.type === 'Variant';
  });
  return styles.reduce(function (files, module) {
    var classes = (0, _utils.generateCSSClasses)(module.body, variants);
    files[module.name + '.elo.css'] = classes.filter(function (selector) {
      return selector.declarations.length > 0;
    }).map(function (selector) {
      var css = (0, _utils.stringifyCSSRule)(selector.declarations, (0, _utils.getModuleName)(module, devMode) + selector.modifier + selector.pseudo, selector.media ? '  ' : '');

      if (selector.media) {
        return '@media ' + selector.media + ' {\n' + css + '\n}';
      }

      return css;
    }).join('\n\n');
    return files;
  }, {});
}

function generateJS(ast, _ref3, adapter) {
  var devMode = _ref3.devMode,
      dynamicImport = _ref3.dynamicImport;
  // TODO: include fragments
  var styles = ast.body.filter(function (node) {
    return node.type === 'Style';
  });
  var variants = ast.body.filter(function (node) {
    return node.type === 'Variant';
  });
  return styles.reduce(function (files, module) {
    var style = generateStyle(module.body);
    var classNameMap = generateClassNameMap(module.body, variants);
    var variantStyleMap = generateVariantStyleMap(module.body, variants);
    files[module.name + '.elo.js'] = adapter({
      style: style,
      variantStyleMap: variantStyleMap,
      moduleName: module.name,
      dynamicImport: dynamicImport,
      classNameMap: classNameMap,
      resetClassName: '_elo_' + module.format,
      className: (0, _utils.getModuleName)(module, devMode),
      variants: variants.reduce(function (flatVariants, variant) {
        flatVariants[variant.name] = variant.body.map(function (variation) {
          return variation.value;
        });
        return flatVariants;
      }, {})
    });
    return files;
  }, {});
}

function generateClassNameMap(nodes, variants) {
  var classes = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var variations = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  var modifier = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : [];
  var nesting = nodes.filter(function (node) {
    return node.type !== 'Declaration';
  });
  var variantOrder = variants.map(function (variant) {
    return variant.name;
  }); // ensure the variant modifier order is always deterministic

  classes[modifier.sort(function (a, b) {
    return variantOrder.indexOf(a[0]) > variantOrder.indexOf(b[0]) ? 1 : -1;
  }).map(function (_ref4) {
    var _ref5 = _slicedToArray(_ref4, 2),
        name = _ref5[0],
        value = _ref5[1];

    return '__' + name + '-' + value;
  }).join('')] = variations;
  nesting.forEach(function (nest) {
    if (nest.property.type === 'Identifier') {
      var variant = variants.find(function (variant) {
        return variant.name === nest.property.value;
      });

      if (variant) {
        if (nest.value.type === 'Identifier') {
          var variation = variant.body.find(function (variant) {
            return variant.value === nest.value.value;
          });

          if (variation) {
            generateClassNameMap(nest.body, variants, classes, _objectSpread({}, variations, _defineProperty({}, variant.name, variation.value)), [].concat(_toConsumableArray(modifier), [[variant.name, variation.value]]));
          }
        }
      } else {// TODO: throw
      }
    }
  });
  return classes;
}

function generateVariantStyleMap(nodes, variants) {
  var styles = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var style = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];
  var modifier = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : [];
  var nesting = nodes.filter(function (node) {
    return node.type !== 'Declaration';
  });
  var variantOrder = variants.map(function (variant) {
    return variant.name;
  });

  if (style.length > 0) {
    styles[modifier.sort(function (a, b) {
      return variantOrder.indexOf(a[0]) > variantOrder.indexOf(b[0]) ? 1 : -1;
    }).map(function (_ref6) {
      var _ref7 = _slicedToArray(_ref6, 2),
          name = _ref7[0],
          value = _ref7[1];

      return '__' + name + '-' + value;
    }).join('')] = style;
  }

  nesting.forEach(function (nest) {
    if (nest.property.type === 'Identifier') {
      var variant = variants.find(function (variant) {
        return variant.name === nest.property.value;
      });

      if (variant) {
        if (nest.value.type === 'Identifier') {
          var variation = variant.body.find(function (variant) {
            return variant.value === nest.value.value;
          });

          if (variation) {
            generateVariantStyleMap(nest.body, variants, styles, generateStyle(nest.body), [].concat(_toConsumableArray(modifier), [[variant.name, variation.value]]));
          }
        }
      } else {// TODO: throw
      }
    }
  });
  return styles;
}

function generateStyle(nodes) {
  var base = nodes.filter(function (node) {
    return node.type === 'Declaration';
  });
  var nestings = nodes.filter(function (node) {
    return node.type !== 'Declaration';
  });
  var declarations = base.filter(function (decl) {
    return decl.dynamic;
  }).map(function (declaration) {
    return {
      property: declaration.property,
      value: declaration.value.value
    };
  });
  var nests = nestings.map(function (nest) {
    if (nest.property.type === 'Variable' && nest.property.environment) {
      if (nest["boolean"]) {
        var pseudoClass = (0, _utils.isPseudoClass)(nest.property.value);
        var pseudoElement = (0, _utils.isPseudoElement)(nest.property.value);

        if (pseudoClass || pseudoElement) {
          return {
            property: (pseudoElement ? '::' : ':') + nest.property.value,
            value: generateStyle(nest.body)
          };
        }
      }

      if ((0, _utils.isMediaQuery)(nest.property.value)) {
        return {
          property: '@media ' + (0, _utils.generateCSSMediaQueryFromNode)(nest.value.value, nest.property.value, nest.operator),
          value: generateStyle(nest.body)
        };
      }
    }
  }).filter(function (nesting) {
    return nesting && nesting.value.length > 0;
  });
  return [].concat(_toConsumableArray(declarations), _toConsumableArray(nests));
}